﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ScatterSearch.TravelingSalesman {
  /// <summary>
  /// An operator that performs diversity calculation between permutations.
  /// </summary>
  [Item("PermutationDiversityCalculator", "An operator that performs diversity calculation between permutations.")]
  [StorableClass]
  public sealed class PermutationDiversityCalculator : DiversityCalculator {
    [StorableConstructor]
    private PermutationDiversityCalculator(bool deserializing) : base(deserializing) { }
    private PermutationDiversityCalculator(PermutationDiversityCalculator original, Cloner cloner) : base(original, cloner) { }
    public PermutationDiversityCalculator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PermutationDiversityCalculator(this, cloner);
    }

    protected override int CalculateDiversity(IItem left, IItem right) {
      var v1 = left as Permutation;
      var v2 = right as Permutation;
      int diversity = 0;
      if (v1 != null && v2 != null)
        for (int i = 0; i < v1.Length; i++) {
          var node = v2.Select((x, index) => new { Value = x, ValueIndex = index }).First(x => x.Value == v1[i]);
          var pred = v2[(node.ValueIndex - 1 + v2.Length) % v2.Length];
          var succ = v2[(node.ValueIndex + 1 % v2.Length) % v2.Length];
          if (new[] { pred, succ }.All(x => x != v1[(i + 1) % v1.Length])) diversity++;
        }
      return diversity;
    }
  }
}
